@@ -352,7 +352,7 @@ void print_tok_vec(std::vector<float> &embd)
             {
                 fileformatmeta->model_architecture = GGUFArch::ARCH_RWKV;
             }
-            else if(modelarch=="glm4")
+            else if(modelarch=="glm4" || modelarch=="glm4moe")
             {
                 fileformatmeta->model_architecture = GGUFArch::ARCH_GLM4;
             }
